#ifndef SMOP_S1P_OO_H
void smop_s1p_oo_init(SMOP__Object* interpreter);
void smop_pureprototypehow_init(SMOP__Object* interpreter);
void smop_s1p_oo_attribute_init(SMOP__Object* interpreter);
void smop_s1p_oo_destr(SMOP__Object* interpreter);
void smop_s1p_oo_attribute_destr(SMOP__Object* interpreter);
void smop_pureprototypehow_destr(SMOP__Object* interpreter);
#endif
