/* DO NOT EDIT: This file is automatically generated by Cabal */

/* package base-4.2.0.0 */
#define MIN_VERSION_base(major1,major2,minor) \
  (major1) <  4 || \
  (major1) == 4 && (major2) <  2 || \
  (major1) == 4 && (major2) == 2 && (minor) <= 0

/* package containers-0.3.0.0 */
#define MIN_VERSION_containers(major1,major2,minor) \
  (major1) <  0 || \
  (major1) == 0 && (major2) <  3 || \
  (major1) == 0 && (major2) == 3 && (minor) <= 0

/* package parsec-2.1.0.1 */
#define MIN_VERSION_parsec(major1,major2,minor) \
  (major1) <  2 || \
  (major1) == 2 && (major2) <  1 || \
  (major1) == 2 && (major2) == 1 && (minor) <= 0

/* package utf8-string-0.3.4 */
#define MIN_VERSION_utf8_string(major1,major2,minor) \
  (major1) <  0 || \
  (major1) == 0 && (major2) <  3 || \
  (major1) == 0 && (major2) == 3 && (minor) <= 4

